﻿;================================================================= 
; Library:           ManagedMessageBox 2.0 
; Author:            Lloyd Gallant (netmaestro) 
; Date:              January 9, 2008 
; Target OS:         Microsoft Windows All 
; Target Compiler:   PureBasic 4.0 and later 
; License:           Free, unrestricted, no warranty 
; 
; Last Update:       May 28, 2008 
; 
; Usage: Filling the MMMBDATA structure 
; 
; Use only the programmer-selectable options. All fields are optional. 
; If you do not fill x and y for a screenposition, the messagebox will 
; be screencentered. 
; 
; #MMB_ Constant usage: 
; 
;   If you do not fill bkContentType (or use #MMB_NOBACKGROUND) no special 
;   background options will be applied. 
; 
;   To have a background color, fill bkContent with a colorref and fill 
;   bkContentType with #MMB_COLOR 
; 
;   To have a background image, fill bkContent with the #image you want to use 
;   and fill bkContentType with #MMB_IMAGE. 
; 
;   #MMB_TEST is a special case. If you have a text in mind for the messagebox 
;   and you would like to have a background image, it is useful to know what 
;   size the client area of the message box will be. If you fill bkContentType 
;   with #MMB_TEST, the caption of the message box will be ignored and it will 
;   show the width And height. You'd only do this during development so you can 
;   easily create a nice image the right size for the message box. 
; 
; txtColor: Put a colorref here if a text color other than black is desired. 
; 
; delayTime: Leave at 0 for a non-timed message box or put a number of seconds 
; to wait before the default button is auto-selected. 
; 
; defaultButton: choose from a 1-based enumeration of the buttons on your 
; messagebox. For example, if your flags are #MB_ABORTRETRYIGNORE, you will 
; have three buttons. Filling defaultButton with 2 will make RETRY the default 
; choice. 
; 
; modalLevel: For normal application modality, leave at 0. For system-level 
; modality (the box will never allow itself to be minimized), fill with 1. 
; 
; silent: Not for use in icon message boxes (like #MB_ICONERROR, etc.), this 
; flag will remove the "ding" sound made when the message box appears. Fill with 
; 0 for normal sound or 1 to suppress the ding. 
; 
; You should compile ThreadSafe if using this library. 
; 
; Enjoy it! 
; 
;================================================================= 

Enumeration 
  #MMB_NOBACKGROUND 
  #MMB_COLOR 
  #MMB_IMAGE 
  #MMB_TEST 
EndEnumeration 

Structure MMBDATA 
  ;------------------------------------------------- 
  ;    Programmer-Selectable Options 
  ;------------------------------------------------- 
  x.l               ; In, desired x position on screen 
  y.l               ; In, desired y position on screen 
  bkContent.l       ; In, long value representing imageid or colorref depending on type 
  bkContentType.b   ; In, specifies whether bkContent is an image or a colorref 
  txtColor.l        ; In, specifies desired text color for message box 
  delayTime.l       ; In, specifies number of seconds to wait before auto-clicking the default button 
  defaultButton.l   ; In, specifies the default button 
  modalLevel.b      ; In, specifiels modality desired 
  silent.b          ; In, specifies no "ding" when messagebox appears. Only for non-icon messageboxes. 
  ;------------------------------------------------- 
  ;    Variables for Internal Use Only 
  ;------------------------------------------------- 
  hwndMain.l        ; reserved, do not use 
  timerThreadID.l   ; reserved, do not use 
  textGadgetID.l    ; reserved, do not use 
  defaultHwnd.l     ; reserved, do not use 
  defaultCtrlID.l   ; reserved, do not use 
  defaultHwndName.s ; reserved, do not use 
  staticBrush.l     ; reserved, do not use 
  bkImageID.l       ; reserved, do not use 
  cx.l              ; reserved, do not use 
  cy.l              ; reserved, do not use 
EndStructure 

Global mmb_Hook, gMMB_DATA_RESERVED.MMBDATA , mmb_ForcedResult=0 

Procedure MMB_dlgProc(hwnd, msg, wparam, lparam) 
  oldproc = GetProp_(hwnd, "oldproc") 
  Select msg 
  
    Case #WM_CLOSE 
      DestroyWindow_(hwnd) 
      mmb_ForcedResult = 1 

    Case #WM_CTLCOLORSTATIC 
      SetBkMode_(wparam, #TRANSPARENT) 
      SetTextColor_(wParam, gMMB_DATA_RESERVED\txtColor) 
      ProcedureReturn gMMB_DATA_RESERVED\staticBrush 
    
    Case #WM_NCDESTROY 
      RemoveProp_(hwnd, "oldproc") 
      DeleteObject_(gMMB_DATA_RESERVED\staticBrush) 

      If IsImage(gMMB_DATA_RESERVED\bkImageID) 
        FreeImage(gMMB_DATA_RESERVED\bkImageID) 
      EndIf 

      If IsThread(gMMB_DATA_RESERVED\timerThreadID) 
        KillThread(gMMB_DATA_RESERVED\timerThreadID) 
      EndIf 

      If IsGadget(gMMB_DATA_RESERVED\textGadgetID) 
        FreeGadget(gMMB_DATA_RESERVED\textGadgetID) 
      EndIf 
            
  EndSelect 
  
  ProcedureReturn CallWindowProc_(oldproc, hwnd, msg, wparam, lparam) 
  
EndProcedure 

Procedure MMB_GetDefaultHwnd(hwnd, lparam) 
  Static buttoncount = 0 
  If lparam = -1 
    buttoncount = 0 
    ProcedureReturn 0 
  EndIf 
  wc$ = Space(255) 
  GetClassName_(hwnd, @wc$, 250) 
  If UCase(wc$) = "BUTTON" 
    buttoncount+1 
    If buttoncount = gMMB_DATA_RESERVED\defaultButton 
      gMMB_DATA_RESERVED\defaultHwnd = hwnd 
      gMMB_DATA_RESERVED\defaultCtrlID = GetDlgCtrlID_(hwnd) 
      GetWindowText_(hwnd, @wc$,250) 
      gMMB_DATA_RESERVED\defaultHwndName = RemoveString(wc$,"&") 
      ProcedureReturn 0 
    EndIf 
  EndIf 
  ProcedureReturn 1 
EndProcedure 

Procedure MMB_TimerThread(secondsremaining) 
  t=ElapsedMilliseconds() 
  Repeat 
    Delay(1000) 
    secondsremaining-1 
    SetGadgetText(gMMB_DATA_RESERVED\textGadgetID, gMMB_DATA_RESERVED\defaultHwndName+" = "+Str(secondsremaining)) 
  Until secondsremaining <=0 
  PostMessage_(gMMB_DATA_RESERVED\hwndmain, #WM_SYSCOMMAND, #SC_HOTKEY, gMMB_DATA_RESERVED\hwndmain) 
  Delay(200) 
  PostMessage_(gMMB_DATA_RESERVED\defaultHwnd, #WM_LBUTTONDOWN,0,0) 
  Delay(200) 
  PostMessage_(gMMB_DATA_RESERVED\defaultHwnd, #WM_LBUTTONUP,0,0) 
  Delay(200) 
  If IsWindow_(gMMB_DATA_RESERVED\hwndmain) 
    SendMessage_(gMMB_DATA_RESERVED\hwndmain, #WM_SYSCOMMAND, #SC_CLOSE, 0) 
  EndIf 
EndProcedure 

Procedure MMB_CBTHookProc(nCode, wParam, lParam) 

  Static hwndmain 
  Select nCode 
  
    Case #HCBT_CREATEWND 
      *pcbt.CBT_CREATEWND = lParam 
      *pcs.CREATESTRUCT = *pcbt\lpcs 
      
      Select *pcs\lpszClass 
      
        Case 32770    
          hwndmain = wParam 
          gMMB_DATA_RESERVED\hwndmain = wParam 
  
          If Not (gMMB_DATA_RESERVED\x=0 And gMMB_DATA_RESERVED\y=0) 
            *pcs\x = gMMB_DATA_RESERVED\x    
            *pcs\y = gMMB_DATA_RESERVED\y 
          EndIf    
          If gMMB_DATA_RESERVED\delayTime 
            *pcs\cy + 20 
          EndIf 
          
          gMMB_DATA_RESERVED\cx = *pcs\cx - 2*GetSystemMetrics_(#SM_CXFIXEDFRAME) 
          gMMB_DATA_RESERVED\cy = *pcs\cy - 2*GetSystemMetrics_(#SM_CYFIXEDFRAME)-GetSystemMetrics_(#SM_CYCAPTION) 
          SetProp_(wparam, "oldproc", SetWindowLong_(wparam,#GWL_WNDPROC,@MMB_dlgProc())) 
          
      EndSelect 
      
    Case #HCBT_ACTIVATE 
  
      If wParam = hwndmain 
      
        UnhookWindowsHookEx_(mmb_Hook) 
        
        MMB_GetDefaultHwnd(0, -1) ; clear the static buttoncount var before enumerating the controls 
        EnumChildWindows_(hwndmain, @MMB_GetDefaultHwnd(), 0) 
      
        ext_content = #False 
        
        Select gMMB_DATA_RESERVED\bkContentType 
        
          Case #MMB_COLOR 
            ext_content = #True 
            img = CreateImage(#PB_Any, gMMB_DATA_RESERVED\cx, gMMB_DATA_RESERVED\cy, 32) 
            StartDrawing(ImageOutput(img)) 
              Box(0,0,gMMB_DATA_RESERVED\cx, gMMB_DATA_RESERVED\cy, gMMB_DATA_RESERVED\bkContent) 
            StopDrawing() 
            gMMB_DATA_RESERVED\bkImageID = img 
            
          Case #MMB_IMAGE 
            ext_content = #True 
            If IsImage(gMMB_DATA_RESERVED\bkContent) 
              tmp = CopyImage(gMMB_DATA_RESERVED\bkContent, #PB_Any) 
              ResizeImage(tmp,gMMB_DATA_RESERVED\cx,gMMB_DATA_RESERVED\cy) 
              gMMB_DATA_RESERVED\bkImageID = tmp 
            Else 
              ext_content = #False 
            EndIf 
            
          Case #MMB_TEST 
            ext_content = #True 
            img = CreateImage(#PB_Any, gMMB_DATA_RESERVED\cx, gMMB_DATA_RESERVED\cy, 32) 
            StartDrawing(ImageOutput(img)) 
              Box(0,0,gMMB_DATA_RESERVED\cx, gMMB_DATA_RESERVED\cy, GetSysColor_(#COLOR_BTNFACE)) 
            StopDrawing()        
            gMMB_DATA_RESERVED\bkImageID = img  
            tmp$ = "w"+Str(gMMB_DATA_RESERVED\cx)+",h"+Str(gMMB_DATA_RESERVED\cy) 
            SetWindowText_(hwndmain, @tmp$) 
                        
          Case #MMB_NOBACKGROUND 
            ext_content = #False 
            
        EndSelect 
        
        If ext_content 
          CreateGadgetList(wParam) 
          ig = ImageGadget(#PB_Any, 0,0,0,0,ImageID(gMMB_DATA_RESERVED\bkImageID)) 
          SetWindowPos_(GadgetID(ig), #HWND_BOTTOM,0,0,0,0,#SWP_NOMOVE|#SWP_NOSIZE) 
          SetWindowLong_(GadgetID(ig),#GWL_STYLE,GetWindowLong_(GadgetID(ig),#GWL_STYLE)|#WS_CLIPSIBLINGS) 
          DisableGadget(ig, #True) 
          txthwnd = GetDlgItem_(wparam, $FFFF)        
          GetWindowRect_(txthwnd, @wr.RECT) 
          gMMB_DATA_RESERVED\staticBrush = CreatePatternBrush_(ImageID(gMMB_DATA_RESERVED\bkImageID)) 
        EndIf 
        
        If gMMB_DATA_RESERVED\delayTime 
          If Not ext_content 
            CreateGadgetList(wParam) 
          EndIf 
          gMMB_DATA_RESERVED\textGadgetID = TextGadget(#PB_Any, 0,gMMB_DATA_RESERVED\cy-22,gMMB_DATA_RESERVED\cx,20, gMMB_DATA_RESERVED\defaultHwndName+" = "+Str(gMMB_DATA_RESERVED\delayTime),#SS_CENTER) 
          exstyle = GetWindowLong_(gMMB_DATA_RESERVED\textGadgetID, #GWL_EXSTYLE)    
          SetWindowLong_(gMMB_DATA_RESERVED\textGadgetID, #GWL_EXSTYLE, exstyle | #WS_EX_NOPARENTNOTIFY) 
          If Not IsThread(gMMB_DATA_RESERVED\timerThreadID)          
            gMMB_DATA_RESERVED\timerThreadID = CreateThread(@MMB_TimerThread(), gMMB_DATA_RESERVED\delayTime) 
          EndIf 
        EndIf 
        
        ProcedureReturn 0 
      EndIf 
      
  EndSelect 
  
  ProcedureReturn CallNextHookEx_(mmb_Hook, nCode, wParam, lParam) 
  
 EndProcedure 

ProcedureDLL.l ManagedMessageBox(title$, text$, *mmb_data_in.MMBDATA, flags=0) 

  With gMMB_DATA_RESERVED 
    \x               = *mmb_data_in\x          
    \y               = *mmb_data_in\y              
    \bkContent       = *mmb_data_in\bkContent      
    \bkContentType   = *mmb_data_in\bkContentType    
    \txtColor        = *mmb_data_in\txtColor      
    \delayTime       = *mmb_data_in\delayTime        
    \defaultButton   = *mmb_data_in\defaultButton    
    \modalLevel      = *mmb_data_in\modalLevel      
    \silent          = *mmb_data_in\silent 
    \hwndMain        = 0 
    \timerThreadID   = 0 
    \textGadgetID    = 0    
    \defaultHwnd     = 0      
    \defaultCtrlID   = 0 
    \defaultHwndName = "" 
    \staticBrush     = 0    
    \bkImageID       = 0      
    \cx              = 0 
    \cy              = 0 
  EndWith 
  
  If gMMB_DATA_RESERVED\delayTime 
    gMMB_DATA_RESERVED\modalLevel = 1 
  EndIf 
  
  Select gMMB_DATA_RESERVED\defaultButton 
    Case 0 
      gMMB_DATA_RESERVED\defaultButton = 1 
      flags = flags|#MB_DEFBUTTON1 
    Case 2 
      flags = flags|#MB_DEFBUTTON2 
    Case 3 
      flags = flags|#MB_DEFBUTTON3 
    Default 
      flags = flags|#MB_DEFBUTTON1 
  EndSelect 
  If gMMB_DATA_RESERVED\BkContent 
    If Not gMMB_DATA_RESERVED\BkContentType 
      gMMB_DATA_RESERVED\BkContentType = #MMB_COLOR 
    EndIf 
  EndIf  
  If gMMB_DATA_RESERVED\modalLevel 
    flags = flags | #MB_SYSTEMMODAL 
  EndIf 
  If gMMB_DATA_RESERVED\silent 
    flags = flags | $C0 
  EndIf 
  mmb_Hook = SetWindowsHookEx_(#WH_CBT, @MMB_CBTHookProc(), #Null, GetCurrentThreadId_()) 
  result = MessageBox_(0, text$, title$, flags) 
  If mmb_ForcedResult 
    result = gMMB_DATA_RESERVED\defaultCtrlID 
    mmb_ForcedResult = 0 
  EndIf 
  
  ProcedureReturn result 
  
EndProcedure 

; IDE Options = PureBasic 4.20 (Windows - x86)
; Folding = -
; EnableXP
; CompileSourceDirectory